<?php

namespace Dmatthew\Brand\Model\Brand;

use Dmatthew\Brand\Model\Brand;
use Magento\UrlRewrite\Service\V1\Data\UrlRewrite;
use Magento\UrlRewrite\Model\OptionProvider;
use Magento\UrlRewrite\Model\UrlFinderInterface;
use Dmatthew\Brand\Model\BrandUrlRewriteGenerator;
use Dmatthew\Brand\Model\BrandUrlPathGenerator;
use Magento\UrlRewrite\Service\V1\Data\UrlRewriteFactory;
use Magento\Store\Model\StoreManagerInterface;

/**
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class CurrentUrlRewritesRegenerator
{
    /** @var Brand */
    protected $brand;

    /** @var UrlFinderInterface */
    protected $urlFinder;

    /** @var BrandUrlPathGenerator */
    protected $brandUrlPathGenerator;

    /** @var UrlRewriteFactory */
    protected $urlRewriteFactory;

    /**
     * @param UrlFinderInterface $urlFinder
     * @param BrandUrlPathGenerator $brandUrlPathGenerator
     * @param UrlRewriteFactory $urlRewriteFactory
     */
    public function __construct(
        UrlFinderInterface $urlFinder,
        BrandUrlPathGenerator $brandUrlPathGenerator,
        UrlRewriteFactory $urlRewriteFactory
    ) {
        $this->urlFinder = $urlFinder;
        $this->brandUrlPathGenerator = $brandUrlPathGenerator;
        $this->urlRewriteFactory = $urlRewriteFactory;
    }

    /**
     * Generate list based on current rewrites
     *
     * @param int $storeId
     * @param Brand $brand
     * @return UrlRewrite[]
     */
    public function generate($storeId, Brand $brand)
    {
        $this->brand = $brand;

        $currentUrlRewrites = $this->urlFinder->findAllByData(
            [
                UrlRewrite::STORE_ID => $storeId,
                UrlRewrite::ENTITY_ID => $this->brand->getId(),
                UrlRewrite::ENTITY_TYPE => BrandUrlRewriteGenerator::ENTITY_TYPE,
            ]
        );

        $urlRewrites = [];
        foreach ($currentUrlRewrites as $currentUrlRewrite) {
            $url = $currentUrlRewrite->getIsAutogenerated()
                ? $this->generateForAutogenerated($currentUrlRewrite, $storeId)
                : $this->generateForCustom($currentUrlRewrite, $storeId);
            $urlRewrites = array_merge($urlRewrites, $url);
        }

        $this->brand = null;
        return $urlRewrites;
    }

    /**
     * @param UrlRewrite $url
     * @param int $storeId
     * @return array
     */
    protected function generateForAutogenerated($url, $storeId)
    {
        if (!$this->brand->getData('save_rewrites_history')) {
            return [];
        }
        $targetPath = $this->brandUrlPathGenerator->getUrlPathWithSuffix($this->brand, $storeId);
        if ($url->getRequestPath() === $targetPath) {
            return [];
        }
        return [
            $this->urlRewriteFactory->create()
                ->setEntityType(BrandUrlRewriteGenerator::ENTITY_TYPE)
                ->setEntityId($this->brand->getId())
                ->setRequestPath($url->getRequestPath())
                ->setTargetPath($targetPath)
                ->setRedirectType(OptionProvider::PERMANENT)
                ->setStoreId($storeId)
                ->setDescription($url->getDescription())
                ->setIsAutogenerated(0)
                ->setMetadata($url->getMetadata())
        ];
    }

    /**
     * @param UrlRewrite $url
     * @param int $storeId
     * @return array
     */
    protected function generateForCustom($url, $storeId)
    {
        $targetPath = $url->getRedirectType()
            ? $this->brandUrlPathGenerator->getUrlPathWithSuffix($this->brand, $storeId)
            : $url->getTargetPath();
        if ($url->getRequestPath() === $targetPath) {
            return [];
        }
        return [
            $this->urlRewriteFactory->create()
                ->setEntityType(BrandUrlRewriteGenerator::ENTITY_TYPE)
                ->setEntityId($this->brand->getId())
                ->setRequestPath($url->getRequestPath())
                ->setTargetPath($targetPath)
                ->setRedirectType($url->getRedirectType())
                ->setStoreId($storeId)
                ->setDescription($url->getDescription())
                ->setIsAutogenerated(0)
                ->setMetadata($url->getMetadata())
        ];
    }
}
